@@ -21,9 +21,10 @@ except ImportError:
     from django.utils.encoding import smart_str as smart_bytes
     from django.utils.encoding import smart_unicode as smart_text
 
-from django.utils.datastructures import SortedDict
-from django.core.exceptions import ImproperlyConfigured
 from django.conf import settings
+from django.core.cache.backends.base import get_key_func
+from django.core.exceptions import ImproperlyConfigured
+from django.utils.datastructures import SortedDict
 
 try:
     from django.core.cache.backends.base import DEFAULT_TIMEOUT
@@ -33,12 +34,20 @@ except ImportError:
 from redis.exceptions import ConnectionError
 from redis.exceptions import ResponseError
 
-from redis_cache.util import CacheKey
-from redis_cache.util import integer_types
+# Compatibility with redis-py 2.10.x+
+
+import socket
+
+try:
+    from redis.exceptions import TimeoutError
+    _main_exceptions = (TimeoutError, ConnectionError, socket.timeout)
+except ImportError:
+    _main_exceptions = (ConnectionError, socket.timeout)
+
+from redis_cache.util import CacheKey, integer_types
 from redis_cache.exceptions import ConnectionInterrupted
 from redis_cache import pool
 
-
 class DefaultClient(object):
     def __init__(self, server, params, backend):
         self._pickle_version = -1
@@ -46,6 +55,8 @@ class DefaultClient(object):
         self._server = server
         self._params = params
 
+        self.reverse_key = get_key_func(params.get('REVERSE_KEY_FUNCTION') or 'redis_cache.util.default_reverse_key')
+
         if not self._server:
             raise ImproperlyConfigured("Missing connections string")
 
@@ -154,8 +165,8 @@ class DefaultClient(object):
                         return self.delete(key, client=client)
 
                 return client.set(key, value)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def incr_version(self, key, delta=1, version=None, client=None):
         """
@@ -174,8 +185,8 @@ class DefaultClient(object):
 
         try:
             ttl = client.ttl(old_key)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
         if value is None:
             raise ValueError("Key '%s' not found" % key)
@@ -210,8 +221,8 @@ class DefaultClient(object):
 
         try:
             value = client.get(key)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
         if value is None:
             return default
@@ -227,8 +238,8 @@ class DefaultClient(object):
 
         try:
             return client.delete(self.make_key(key, version=version))
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def delete_pattern(self, pattern, version=None, client=None):
         """
@@ -244,8 +255,8 @@ class DefaultClient(object):
 
             if keys:
                 return client.delete(*keys)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def delete_many(self, keys, version=None, client=None):
         """
@@ -261,8 +272,8 @@ class DefaultClient(object):
         keys = [self.make_key(k, version=version) for k in keys]
         try:
             return client.delete(*keys)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def clear(self, client=None):
         """
@@ -313,8 +324,8 @@ class DefaultClient(object):
 
         try:
             results = client.mget(*new_keys)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
         for key, value in zip(new_keys, results):
             if value is None:
@@ -338,8 +349,8 @@ class DefaultClient(object):
             for key, value in data.items():
                 self.set(key, value, timeout, version=version, client=pipeline)
             pipeline.execute()
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def _incr(self, key, delta=1, version=None, client=None):
         if client is None:
@@ -364,8 +375,8 @@ class DefaultClient(object):
                 value = self.get(key, version=version, client=client) + delta
                 self.set(key, value, version=version, timeout=timeout,
                          client=client)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
         return value
 
@@ -408,8 +419,8 @@ class DefaultClient(object):
         key = self.make_key(key, version=version)
         try:
             return client.exists(key)
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def iter_keys(self, search, itersize=None, client=None, version=None):
         """
@@ -428,7 +439,7 @@ class DefaultClient(object):
 
             for item in data:
                 item = smart_text(item)
-                yield item.split(":", 2)[2]
+                yield self.reverse_key(item)
 
             if cursor == b"0":
                 break
@@ -447,9 +458,9 @@ class DefaultClient(object):
         pattern = self.make_key(search, version=version)
         try:
             encoding_map = [smart_text(k) for k in client.keys(pattern)]
-            return [k.split(":", 2)[2] for k in encoding_map]
-        except ConnectionError:
-            raise ConnectionInterrupted(connection=client)
+            return [self.reverse_key(k) for k in encoding_map]
+        except _main_exceptions as e:
+            raise ConnectionInterrupted(connection=client, parent=e)
 
     def make_key(self, key, version=None):
         if isinstance(key, CacheKey):
